/*
   Test Arduino Manager for iPad / iPhone / Mac

   A simple test program to show the Arduino Manager
   features.

   Author: Fabrizio Boco - fabboco@gmail.com

   Version: 1.0

   08/13/2023

   All rights reserved

*/

/*

   AMController libraries, example sketches (The Software) and the related documentation (The Documentation) are supplied to you
   by the Author in consideration of your agreement to the following terms, and your use or installation of The Software and the use of The Documentation
   constitutes acceptance of these terms.
   If you do not agree with these terms, please do not use or install The Software.
   The Author grants you a personal, non-exclusive license, under authors copyrights in this original software, to use The Software.
   Except as expressly stated in this notice, no other rights or licenses, express or implied, are granted by the Author, including but not limited to any
   patent rights that may be infringed by your derivative works or by other works in which The Software may be incorporated.
   The Software and the Documentation are provided by the Author on an AS IS basis.  THE AUTHOR MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT
   LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND OPERATION
   ALONE OR IN COMBINATION WITH YOUR PRODUCTS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
   REPRODUCTION AND MODIFICATION OF THE SOFTWARE AND OR OF THE DOCUMENTATION, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
   STRICT LIABILITY OR OTHERWISE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <Servo.h>
#include <AM_UnoR4WiFi.h>
#include "Arduino_LED_Matrix.h"


#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>


#define SD_SELECT 10

/*

   WIFI Library configuration

*/
IPAddress ip(a, b, c, d);
IPAddress dns(8, 8, 8, 8);
IPAddress gateway(e, f, g, h);
IPAddress subnet(255, 255, 255, 0);

char ssid[] = SECRET_SSID;  // your network SSID (name) e.g. "MYNETWORK"
char pass[] = SECRET_PASS;  // your network password e.g. "MYPASSWORD"

int status = WL_IDLE_STATUS;

WiFiServer server(80);

/**

   Other initializations

*/

#define PUSH_BUTTON_PIN 7
#define YELLOW_LED_PIN 8
#define RED_LED_PIN DAC
#define POTENTIOMETER_PIN A2
#define DHTPIN 5                            // Digital pin connected to the DHT sensor
#define DHTTYPE DHT22                       // DHT 22 (AM2302)
#define TEMPERATURE_SAMPLE_PERIOD 60000     // [ms]
#define TEMPERATURE_STORE_PERIOD 2 * 60000  // 5 minutes [ms]

DHT_Unified dht(DHTPIN, DHTTYPE);

bool pushButtonPressed = false;
float temperature;
float humidity;
unsigned long lastTemperatureSampleTime = TEMPERATURE_SAMPLE_PERIOD;
unsigned long lastTemperatureStoreTime = TEMPERATURE_STORE_PERIOD;
int potentiometer;

ArduinoLEDMatrix matrix;

const uint32_t AM_Clear[] = {};

const uint32_t AM_Logo[] = {
  0x64196395,
  0x5f499419,
  0x41941941,
  66
};

/*

   Callbacks Prototypes

*/
void doWork();
void doSync();
void processIncomingMessages(char *variable, char *value);
void processOutgoingMessages();
void processAlarms(char *variable);
void deviceConnected();
void deviceDisconnected();

/*

   AMController Library initialization

*/
#ifdef ALARMS_SUPPORT
AMController amController(&server, &doWork, &doSync, &processIncomingMessages, &processOutgoingMessages, &processAlarms, &deviceConnected, &deviceDisconnected);
#else
AMController amController(&server, &doWork, &doSync, &processIncomingMessages, &processOutgoingMessages, &deviceConnected, &deviceDisconnected);
#endif

void setup() {

  Serial.begin(115200);
  Serial.println("Starting...");

#if (defined(SD_SUPPORT))
  Serial.println("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(SD_SELECT)) {
    Serial.println("SD card failed, or not present");
  } else {
    Serial.println("SD card initialized");
  }

#endif

  // attempt to connect to Wifi network
  Serial.println("Configuring network...");
  WiFi.config(ip, dns, gateway, subnet);
  Serial.println("Wifi Configured");

  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);

    // Connect to WPA/WPA2 network. Change this line if using open or WEP network
    status = WiFi.begin(ssid, pass);

    // wait 1 seconds for connection:
    delay(1000);
  }

  // print your WiFi shields IP address

  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

#if defined(ALARMS_SUPPORT)

  // Set a new NTP Server
  //
  // Choose your NTP Server here: www.pool.ntp.org
  //
  amController.setNTPServerAddress(IPAddress(162, 159, 200, 123));

#endif

  /**

    Other initializations

  */

  // Initialize built in LED
  pinMode(LED_BUILTIN, OUTPUT);

  // Initialize Push Button
  pinMode(PUSH_BUTTON_PIN, INPUT);

  // Initialize Yellow LED
  pinMode(YELLOW_LED_PIN, OUTPUT);
  digitalWrite(YELLOW_LED_PIN, LOW);

  // Initialize Red LED and DAC
  analogWriteResolution(10);
  analogWrite(RED_LED_PIN, 0);

  // Initialize potentiometer
  analogReadResolution(10);
  pinMode(POTENTIOMETER_PIN, INPUT);


  // Initialize the temperature / humidity sensor
  dht.begin();

  // Initialize the graph labels

  // Delete all the storage files
  // amController.sdPurgeLogData("Graph_H");
  // amController.sdPurgeLogData("Graph_TC");
  // amController.sdPurgeLogData("Graph_TF");
  // amController.sdPurgeLogData("Graph_H");

  initializeOrCleanLog();

  // Initialize LED Matrix
  matrix.begin();

  Serial.println("Device Ready");
}

/**

   Standard loop function

*/
void loop() {

  amController.loop(200);
}

/**


  This function is called periodically and its equivalent to the standard loop() function

*/
void doWork() {
  //Serial.println("doWork");

  if ((millis() - lastTemperatureSampleTime) > TEMPERATURE_SAMPLE_PERIOD) {
    lastTemperatureSampleTime = millis();

    Serial.println("Measuring...");
    sensors_event_t event;
    dht.temperature().getEvent(&event);
    if (isnan(event.temperature)) {
      Serial.println(F("Error reading temperature!"));
    } else {
      temperature = event.temperature;
    }
    // Get humidity event and print its value.
    dht.humidity().getEvent(&event);
    if (isnan(event.relative_humidity)) {
      Serial.println(F("Error reading humidity!"));
    } else {
      humidity = event.relative_humidity;
    }
  }

  if ((millis() - lastTemperatureStoreTime) > TEMPERATURE_STORE_PERIOD) {
    lastTemperatureStoreTime = millis();

    Serial.println("Storing...");
    unsigned long now = amController.now();
    amController.sdLog("Graph_TC", now, temperature);
    amController.sdLog("Graph_TF", now, temperature * 1.8 + 32);
    amController.sdLog("Graph_H", now, humidity);
    initializeOrCleanLog();
  }

  potentiometer = analogRead(POTENTIOMETER_PIN);
  pushButtonPressed = digitalRead(PUSH_BUTTON_PIN);
}


/**


  This function is called when the ios device connects and needs to initialize the position of switches and knobs

*/
void doSync() {
  // Serial.println("doSync");
  amController.writeMessage("Yellow_LED", digitalRead(YELLOW_LED_PIN));
  amController.writeMessage("Red_LED", analogRead(RED_LED_PIN));
}

/**


  This function is called when a new message is received from the iOS device

*/
void processIncomingMessages(char *variable, char *value) {
  // Serial.print(variable);
  // Serial.print(" : ");
  // Serial.println(value);

  if (strcmp(variable, "Yellow_LED") == 0) {
    digitalWrite(YELLOW_LED_PIN, atoi(value));
  }

  if (strcmp(variable, "Show") == 0) {
    if (atoi(value) == 1) {
      matrix.loadFrame(AM_Logo);
    } else {
      matrix.loadFrame(AM_Clear);
    }
  }

  if (strcmp(variable, "Red_LED") == 0) {
    analogWrite(RED_LED_PIN, atoi(value));
  }
}

/**


  This function is called periodically and messages can be sent to the iOS device

*/
void processOutgoingMessages() {
  amController.writeMessage("T", temperature);
  amController.writeMessage("H", humidity);
  amController.writeMessage("Pot", potentiometer);
  amController.writeMessage("Button", pushButtonPressed);
}

/**


  This function is called when a Alarm is fired

*/
void processAlarms(char *alarm) {
  //Serial.print(alarm);
  //Serial.println(" fired");
}

/**


  This function is called when the iOS device connects

*/
void deviceConnected() {
  //Serial.println("Device connected");
  amController.writeTxtMessage("Msg", "Hello, this is your Arduino R4 WiFi board");
  digitalWrite(LED_BUILTIN, HIGH);
}

/**


  This function is called when the iOS device disconnects

*/
void deviceDisconnected() {
  //Serial.println("Device disconnected");
  digitalWrite(LED_BUILTIN, LOW);
}

/**

  Auxiliary functions

*/
void initializeOrCleanLog() {
  uint32_t size = amController.sdFileSize("Graph_TC");

  if (size == 0 || size > 10480) {
    Serial.println("Initialize Logging files");
    amController.sdPurgeLogData("Graph_TC");
    amController.sdPurgeLogData("Graph_TF");
    amController.sdLogLabels("Graph_TC", "T [C]");
    amController.sdLogLabels("Graph_TF", "T [F]");

    amController.sdPurgeLogData("Graph_H");
    amController.sdLogLabels("Graph_H", "H [%]");
  }
}